@@ -62,21 +62,20 @@ struct aws_s3_auto_ranged_put {
 
     /* Members to only be used when the mutex in the base type is locked. */
     struct {
-        /* Array list of `struct aws_string *`. */
-        struct aws_array_list etag_list;
-
-        /* Very similar to the etag_list used in complete_multipart_upload to create the XML payload. Each part will set
-         * the corresponding index to its checksum result. */
-        struct aws_array_list encoded_checksum_list;
+        /* Array list of `struct aws_s3_mpu_part_info *`
+         * Info about each part, that we need to remember for CompleteMultipartUpload.
+         * This is updated as we upload each part.
+         * If resuming an upload, we first call ListParts and store the details
+         * of previously uploaded parts here. In this case, the array may start with gaps
+         * (e.g. if parts 1 and 3 were previously uploaded, but not part 2). */
+        struct aws_array_list part_list;
 
         struct aws_s3_paginated_operation *list_parts_operation;
         struct aws_string *list_parts_continuation_token;
 
         /* Number of parts we've started work on */
-        uint32_t num_parts_sent;
-        /* Number of "sent" parts we've finished reading the body for
-         * (does not include skipped parts in the case of pause/resume) */
-        uint32_t num_parts_read;
+        uint32_t num_parts_started;
+        /* Number of parts we've started, and we have no more work to do */
         uint32_t num_parts_completed;
         uint32_t num_parts_successful;
         uint32_t num_parts_failed;
@@ -85,6 +84,11 @@ struct aws_s3_auto_ranged_put {
          * work to do*/
         uint32_t num_parts_noop;
 
+        /* Number of parts we've started, but they're not done reading from stream yet.
+         * Though reads are serial (only 1 part can be reading from stream at a time)
+         * we may queue up more to minimize delays between each read. */
+        uint32_t num_parts_pending_read;
+
         struct aws_http_headers *needed_response_headers;
 
         /* Whether body stream is exhausted. */
