@@ -145,8 +145,8 @@ struct aws_s3_meta_request {
     aws_s3_meta_request_finish_fn *finish_callback;
     aws_s3_meta_request_shutdown_fn *shutdown_callback;
     aws_s3_meta_request_progress_fn *progress_callback;
-
     aws_s3_meta_request_telemetry_fn *telemetry_callback;
+    aws_s3_meta_request_upload_review_fn *upload_review_callback;
 
     /* Customer specified callbacks to be called by our specialized callback to calculate the response checksum. */
     aws_s3_meta_request_headers_callback_fn *headers_user_callback_after_checksum;
@@ -217,6 +217,13 @@ struct aws_s3_meta_request {
     struct aws_s3_checksum *meta_request_level_running_response_sum;
 };
 
+/* Info for each part, that we need to remember until we send CompleteMultipartUpload */
+struct aws_s3_mpu_part_info {
+    uint64_t size;
+    struct aws_string *etag;
+    struct aws_byte_buf checksum_base64;
+};
+
 AWS_EXTERN_C_BEGIN
 
 /* Initialize the base meta request structure. */
