@@ -14,7 +14,6 @@
 
 /* TODO: better logging of steps */
 
-static const struct aws_byte_cursor s_upload_id = AWS_BYTE_CUR_INIT_FROM_STRING_LITERAL("UploadId");
 static const size_t s_complete_multipart_upload_init_body_size_bytes = 512;
 static const size_t s_abort_multipart_upload_init_body_size_bytes = 512;
 /* For unknown length body we no longer know the number of parts. to avoid
@@ -24,12 +23,19 @@ static const size_t s_abort_multipart_upload_init_body_size_bytes = 512;
  */
 static const uint32_t s_unknown_length_default_num_parts = 32;
 
-/* For unknown length body we dont know how many parts we'll end up with.
- * We optimistically schedule upload requests, but cap the number of requests
- * being prepared by bellow number to avoid unknown length request dominating the
- * queue.
- * TODO: this value needs further benchmarking.*/
-static const uint32_t s_unknown_length_max_optimistic_prepared_parts = 5;
+/* Max number of parts (per meta-request) that can be: "started, but not done reading from stream".
+ * Though reads are serial (only 1 part can be reading from stream at a time)
+ * we may queue up more to minimize delays between each read.
+ *
+ * If this number is too low, there could be an avoidable delay between each read
+ * (meta-request ready for more work, but client hasn't run update and given it more work yet)
+ *
+ * If this number is too high, early meta-requests could hog all the "work tokens"
+ * (1st meta-request as queue of 100 "work tokens" that it needs to read
+ * the stream for, while later meta-requests are doing nothing waiting for work tokens)
+ *
+ * TODO: this value needs further benchmarking. */
+static const uint32_t s_max_parts_pending_read = 5;
 
 static const struct aws_byte_cursor s_create_multipart_upload_copy_headers[] = {
     AWS_BYTE_CUR_INIT_FROM_STRING_LITERAL("x-amz-server-side-encryption-customer-algorithm"),
@@ -112,10 +118,21 @@ static int s_s3_auto_ranged_put_pause(
     struct aws_s3_meta_request *meta_request,
     struct aws_s3_meta_request_resume_token **resume_token);
 
-static bool s_process_part_info(const struct aws_s3_part_info *info, void *user_data) {
+static int s_process_part_info_synced(const struct aws_s3_part_info *info, void *user_data) {
     struct aws_s3_auto_ranged_put *auto_ranged_put = user_data;
+    struct aws_s3_meta_request *meta_request = &auto_ranged_put->base;
 
-    struct aws_string *etag = aws_strip_quotes(auto_ranged_put->base.allocator, info->e_tag);
+    ASSERT_SYNCED_DATA_LOCK_HELD(&auto_ranged_put->base);
+
+    if (info->part_number == 0) {
+        AWS_LOGF_ERROR(
+            AWS_LS_S3_META_REQUEST, "id=%p: ListParts reported Part without valid PartNumber", (void *)meta_request);
+        return aws_raise_error(AWS_ERROR_S3_LIST_PARTS_PARSE_FAILED);
+    }
+
+    struct aws_s3_mpu_part_info *part = aws_mem_calloc(meta_request->allocator, 1, sizeof(struct aws_s3_mpu_part_info));
+    part->size = info->size;
+    part->etag = aws_strip_quotes(meta_request->allocator, info->e_tag);
 
     const struct aws_byte_cursor *checksum_cur = NULL;
     switch (auto_ranged_put->base.checksum_config.checksum_algorithm) {
@@ -138,33 +155,23 @@ static bool s_process_part_info(const struct aws_s3_part_info *info, void *user_
             break;
     }
 
-    /* Parts might be out of order or have gaps in them. array list allows
-     * setting elements outside of current length, but values between old length
-     * and new value will be uninitialized. as a workaround when processing
-     * element make sure to init any new elements to zeroed values. */
-    size_t current_num_parts = aws_array_list_length(&auto_ranged_put->synced_data.etag_list);
-    if (info->part_number > current_num_parts) {
-        struct aws_byte_buf empty_buf = {0};
-        struct aws_string *null_etag = NULL;
-
-        /* Note: using 1 based part nums here to avoid dealing with underflow of
-        unsigned number in case part 1 was not completed, but other parts were. */
-        for (size_t part_num = info->part_number; part_num > current_num_parts; --part_num) {
-            aws_array_list_set_at(&auto_ranged_put->synced_data.encoded_checksum_list, &empty_buf, part_num - 1);
-            aws_array_list_set_at(&auto_ranged_put->synced_data.etag_list, &null_etag, part_num - 1);
-        }
+    if ((checksum_cur != NULL) && (checksum_cur->len > 0)) {
+        aws_byte_buf_init_copy_from_cursor(&part->checksum_base64, auto_ranged_put->base.allocator, *checksum_cur);
     }
 
-    if (checksum_cur) {
-        struct aws_byte_buf checksum_buf;
-        aws_byte_buf_init_copy_from_cursor(&checksum_buf, auto_ranged_put->base.allocator, *checksum_cur);
-        aws_array_list_set_at(
-            &auto_ranged_put->synced_data.encoded_checksum_list, &checksum_buf, info->part_number - 1);
+    /* Parts might be out of order or have gaps in them.
+     * Resize array-list to be long enough to hold this part,
+     * filling any intermediate slots with NULL. */
+    aws_array_list_ensure_capacity(&auto_ranged_put->synced_data.part_list, info->part_number);
+    while (aws_array_list_length(&auto_ranged_put->synced_data.part_list) < info->part_number) {
+        struct aws_s3_mpu_part_info *null_part = NULL;
+        aws_array_list_push_back(&auto_ranged_put->synced_data.part_list, &null_part);
     }
 
-    aws_array_list_set_at(&auto_ranged_put->synced_data.etag_list, &etag, info->part_number - 1);
+    /* Add this part */
+    aws_array_list_set_at(&auto_ranged_put->synced_data.part_list, &part, info->part_number - 1);
 
-    return true;
+    return AWS_OP_SUCCESS;
 }
 
 /*
@@ -258,7 +265,7 @@ static int s_try_init_resume_state_from_persisted_data(
         return aws_raise_error(AWS_ERROR_INVALID_ARGUMENT);
     }
 
-    auto_ranged_put->synced_data.num_parts_sent = 0;
+    auto_ranged_put->synced_data.num_parts_started = 0;
     auto_ranged_put->synced_data.num_parts_completed = 0;
     auto_ranged_put->synced_data.num_parts_noop = 0;
     auto_ranged_put->synced_data.create_multipart_upload_sent = true;
@@ -268,7 +275,7 @@ static int s_try_init_resume_state_from_persisted_data(
     struct aws_s3_list_parts_params list_parts_params = {
         .key = request_path,
         .upload_id = aws_byte_cursor_from_string(auto_ranged_put->upload_id),
-        .on_part = s_process_part_info,
+        .on_part = s_process_part_info_synced,
         .user_data = auto_ranged_put,
     };
 
@@ -378,9 +385,7 @@ struct aws_s3_meta_request *aws_s3_meta_request_auto_ranged_put_new(
     uint32_t initial_num_parts = auto_ranged_put->has_content_length ? num_parts : s_unknown_length_default_num_parts;
 
     aws_array_list_init_dynamic(
-        &auto_ranged_put->synced_data.etag_list, allocator, initial_num_parts, sizeof(struct aws_string *));
-    aws_array_list_init_dynamic(
-        &auto_ranged_put->synced_data.encoded_checksum_list, allocator, initial_num_parts, sizeof(struct aws_byte_buf));
+        &auto_ranged_put->synced_data.part_list, allocator, initial_num_parts, sizeof(struct aws_s3_mpu_part_info *));
 
     if (s_try_init_resume_state_from_persisted_data(allocator, auto_ranged_put, options->resume_token)) {
         goto error_clean_up;
@@ -410,27 +415,21 @@ static void s_s3_meta_request_auto_ranged_put_destroy(struct aws_s3_meta_request
 
     aws_s3_paginated_operation_release(auto_ranged_put->synced_data.list_parts_operation);
 
-    for (size_t etag_index = 0; etag_index < aws_array_list_length(&auto_ranged_put->synced_data.etag_list);
-         ++etag_index) {
-        struct aws_string *etag = NULL;
+    for (size_t part_index = 0; part_index < aws_array_list_length(&auto_ranged_put->synced_data.part_list);
+         ++part_index) {
 
-        aws_array_list_get_at(&auto_ranged_put->synced_data.etag_list, &etag, etag_index);
-        aws_string_destroy(etag);
+        struct aws_s3_mpu_part_info *part;
+        aws_array_list_get_at(&auto_ranged_put->synced_data.part_list, &part, part_index);
+        if (part != NULL) {
+            aws_byte_buf_clean_up(&part->checksum_base64);
+            aws_string_destroy(part->etag);
+            aws_mem_release(auto_ranged_put->base.allocator, part);
+        }
     }
+    aws_array_list_clean_up(&auto_ranged_put->synced_data.part_list);
 
     aws_string_destroy(auto_ranged_put->synced_data.list_parts_continuation_token);
 
-    for (size_t checksum_index = 0;
-         checksum_index < aws_array_list_length(&auto_ranged_put->synced_data.encoded_checksum_list);
-         ++checksum_index) {
-
-        struct aws_byte_buf checksum_buf;
-        aws_array_list_get_at(&auto_ranged_put->synced_data.encoded_checksum_list, &checksum_buf, checksum_index);
-
-        aws_byte_buf_clean_up(&checksum_buf);
-    }
-    aws_array_list_clean_up(&auto_ranged_put->synced_data.etag_list);
-    aws_array_list_clean_up(&auto_ranged_put->synced_data.encoded_checksum_list);
     aws_http_headers_release(auto_ranged_put->synced_data.needed_response_headers);
     aws_mem_release(meta_request->allocator, auto_ranged_put);
 }
@@ -441,22 +440,20 @@ static bool s_should_skip_scheduling_more_parts_based_on_flags(
     const struct aws_s3_auto_ranged_put *auto_ranged_put,
     uint32_t flags) {
 
-    uint32_t num_parts_in_flight =
-        (auto_ranged_put->synced_data.num_parts_sent - auto_ranged_put->synced_data.num_parts_completed);
-
-    /* If the stream is actually async, only allow 1 part in flight at a time.
-     * We need to wait for async read() to complete before calling it again */
+    /* If the stream is actually async, only allow 1 pending-read.
+     * We need to wait for async read() to complete before calling it again. */
     if (auto_ranged_put->base.request_body_async_stream != NULL) {
-        return num_parts_in_flight > 0;
+        return auto_ranged_put->synced_data.num_parts_pending_read > 0;
     }
 
+    /* If this is the conservative pass, only allow 1 pending-read.
+     * Reads are serial anyway, so queuing up a whole bunch isn't necessarily a speedup. */
     if ((flags & AWS_S3_META_REQUEST_UPDATE_FLAG_CONSERVATIVE) != 0) {
-        /* Because uploads must read from their streams serially, we try to limit the amount of in flight
-         * requests for a given multipart upload if we can. */
-        return num_parts_in_flight > 0;
+        return auto_ranged_put->synced_data.num_parts_pending_read > 0;
     }
 
-    return false;
+    /* In all other cases, cap the number of pending-reads to something reasonable */
+    return auto_ranged_put->synced_data.num_parts_pending_read >= s_max_parts_pending_read;
 }
 
 static bool s_s3_auto_ranged_put_update(
@@ -526,17 +523,17 @@ static bool s_s3_auto_ranged_put_update(
             }
 
             bool should_create_next_part_request = false;
-            if (auto_ranged_put->has_content_length &&
-                (auto_ranged_put->synced_data.num_parts_sent < auto_ranged_put->total_num_parts_from_content_length)) {
+            if (auto_ranged_put->has_content_length && (auto_ranged_put->synced_data.num_parts_started <
+                                                        auto_ranged_put->total_num_parts_from_content_length)) {
 
                 /* Check if the etag/checksum list has the result already */
-                int part_index = auto_ranged_put->threaded_update_data.next_part_number - 1;
-                for (size_t etag_index = part_index;
-                     etag_index < aws_array_list_length(&auto_ranged_put->synced_data.etag_list);
-                     ++etag_index) {
-                    struct aws_string *etag = NULL;
+                for (size_t part_index = auto_ranged_put->threaded_update_data.next_part_number - 1;
+                     part_index < aws_array_list_length(&auto_ranged_put->synced_data.part_list);
+                     ++part_index) {
 
-                    if (!aws_array_list_get_at(&auto_ranged_put->synced_data.etag_list, &etag, etag_index) && etag) {
+                    struct aws_s3_mpu_part_info *part;
+                    aws_array_list_get_at(&auto_ranged_put->synced_data.part_list, &part, part_index);
+                    if (part != NULL) {
                         /* part already downloaded, skip it here and prepare will take care of adjusting the buffer */
                         ++auto_ranged_put->threaded_update_data.next_part_number;
 
@@ -563,15 +560,6 @@ static bool s_s3_auto_ranged_put_update(
                     goto has_work_remaining;
                 }
 
-                uint32_t num_parts_not_read =
-                    (auto_ranged_put->synced_data.num_parts_sent - auto_ranged_put->synced_data.num_parts_read);
-
-                /* Because uploads must read from their streams serially, we try to limit the amount of in flight
-                 * requests for a given multipart upload if we can. */
-                if (num_parts_not_read >= s_unknown_length_max_optimistic_prepared_parts) {
-                    goto has_work_remaining;
-                }
-
                 should_create_next_part_request = true;
             }
 
@@ -587,7 +575,8 @@ static bool s_s3_auto_ranged_put_update(
                 request->part_number = auto_ranged_put->threaded_update_data.next_part_number;
 
                 ++auto_ranged_put->threaded_update_data.next_part_number;
-                ++auto_ranged_put->synced_data.num_parts_sent;
+                ++auto_ranged_put->synced_data.num_parts_started;
+                ++auto_ranged_put->synced_data.num_parts_pending_read;
 
                 AWS_LOGF_DEBUG(
                     AWS_LS_S3_META_REQUEST,
@@ -608,7 +597,8 @@ static bool s_s3_auto_ranged_put_update(
                 }
             } else {
                 if ((!auto_ranged_put->synced_data.is_body_stream_at_end) ||
-                    auto_ranged_put->synced_data.num_parts_completed != auto_ranged_put->synced_data.num_parts_sent) {
+                    auto_ranged_put->synced_data.num_parts_completed !=
+                        auto_ranged_put->synced_data.num_parts_started) {
                     goto has_work_remaining;
                 }
             }
@@ -646,7 +636,7 @@ static bool s_s3_auto_ranged_put_update(
 
             /* If the number of parts completed is less than the number of parts sent, then we need to wait until all of
              * those parts are done sending before aborting. */
-            if (auto_ranged_put->synced_data.num_parts_completed < auto_ranged_put->synced_data.num_parts_sent) {
+            if (auto_ranged_put->synced_data.num_parts_completed < auto_ranged_put->synced_data.num_parts_started) {
                 goto has_work_remaining;
             }
 
@@ -748,9 +738,9 @@ static size_t s_compute_request_body_size(const struct aws_s3_meta_request *meta
 
 static int s_verify_part_matches_checksum(
     struct aws_allocator *allocator,
-    struct aws_byte_buf part_body,
+    struct aws_byte_cursor body_cur,
     enum aws_s3_checksum_algorithm algorithm,
-    struct aws_byte_buf part_checksum) {
+    struct aws_byte_cursor part_checksum) {
     AWS_PRECONDITION(allocator);
 
     if (algorithm == AWS_SCA_NONE) {
@@ -765,7 +755,6 @@ static int s_verify_part_matches_checksum(
     struct aws_byte_buf encoded_checksum = {0};
 
     int return_status = AWS_OP_SUCCESS;
-    struct aws_byte_cursor body_cur = aws_byte_cursor_from_buf(&part_body);
 
     size_t encoded_len = 0;
     if (aws_base64_compute_encoded_len(aws_get_digest_size_from_algorithm(algorithm), &encoded_len)) {
@@ -796,7 +785,7 @@ static int s_verify_part_matches_checksum(
         goto on_done;
     }
 
-    if (!aws_byte_buf_eq(&encoded_checksum, &part_checksum)) {
+    if (!aws_byte_cursor_eq_byte_buf(&part_checksum, &encoded_checksum)) {
         AWS_LOGF_ERROR(
             AWS_LS_S3_META_REQUEST, "Failed to resume upload. Checksum for previously uploaded part does not match");
         return_status = aws_raise_error(AWS_ERROR_S3_RESUMED_PART_CHECKSUM_MISMATCH);
@@ -816,6 +805,8 @@ struct aws_s3_skip_parts_from_stream_job {
     uint32_t part_index;
     uint32_t skip_until_part_number;
     struct aws_byte_buf temp_body_buf;
+    /* non-owning reference to info about part we're currently skipping */
+    struct aws_s3_mpu_part_info *part_being_skipped;
     /* repeated async step: read each part we're skipping */
     struct aws_future_bool *asyncstep_read_each_part;
     /* future to set when this job completes */
@@ -892,7 +883,29 @@ static void s_skip_parts_from_stream_loop(void *user_data) {
 
         /* kick off an async read if none are pending */
         if (skip_job->asyncstep_read_each_part == NULL) {
+
+            /* Get info about part being skipped.
+             * (Yes it's weird that we keep a pointer to the part_info even after
+             * releasing the lock that protects part_list. But it's the resizable
+             * part_list that needs lock protection. A previously uploaded part_info is const,
+             * and it's on the heap, so it's safe to keep the pointer around) */
+            /* BEGIN CRITICAL SECTION */
+            aws_s3_meta_request_lock_synced_data(meta_request);
+            aws_array_list_get_at(
+                &auto_ranged_put->synced_data.part_list, &skip_job->part_being_skipped, skip_job->part_index);
+            AWS_ASSERT(skip_job->part_being_skipped != NULL);
+            aws_s3_meta_request_unlock_synced_data(meta_request);
+            /* END CRITICAL SECTION */
+
             size_t request_body_size = s_compute_request_body_size(meta_request, skip_job->part_index + 1);
+            if (request_body_size != skip_job->part_being_skipped->size) {
+                error_code = AWS_ERROR_S3_RESUME_FAILED;
+                AWS_LOGF_ERROR(
+                    AWS_LS_S3_META_REQUEST,
+                    "id=%p: Failed resuming upload, previous upload used different part size.",
+                    (void *)meta_request);
+                goto on_done;
+            }
 
             if (temp_body_buf->capacity != request_body_size) {
                 // reinit with correct size
@@ -922,32 +935,29 @@ static void s_skip_parts_from_stream_loop(void *user_data) {
         if (error_code != AWS_ERROR_SUCCESS) {
             AWS_LOGF_ERROR(
                 AWS_LS_S3_META_REQUEST,
-                "id=%p: Failed to resume upload. Input stream read error %d (%s)",
+                "id=%p: Failed resuming upload, error reading request body %d (%s)",
                 (void *)meta_request,
                 error_code,
                 aws_error_str(error_code));
-            error_code = AWS_ERROR_S3_RESUME_FAILED;
             goto on_done;
         }
 
         if (temp_body_buf->len < temp_body_buf->capacity) {
             AWS_LOGF_ERROR(
                 AWS_LS_S3_META_REQUEST,
-                "id=%p: Failed to resume upload. Input steam ended prematurely.",
+                "id=%p: Failed resuming upload, request body smaller than it was during previous upload.",
                 (void *)meta_request);
-            error_code = AWS_ERROR_S3_RESUME_FAILED;
+            error_code = AWS_ERROR_S3_INCORRECT_CONTENT_LENGTH;
             goto on_done;
         }
 
-        struct aws_byte_buf checksum_buf;
-        aws_array_list_get_at(&auto_ranged_put->synced_data.encoded_checksum_list, &checksum_buf, skip_job->part_index);
-
-        // compare skipped checksum to previously uploaded checksum
-        if (checksum_buf.len > 0 && s_verify_part_matches_checksum(
-                                        meta_request->allocator,
-                                        *temp_body_buf,
-                                        meta_request->checksum_config.checksum_algorithm,
-                                        checksum_buf)) {
+        // if previously uploaded part had a checksum, compare it to what we just skipped
+        if (skip_job->part_being_skipped->checksum_base64.len > 0 &&
+            s_verify_part_matches_checksum(
+                meta_request->allocator,
+                aws_byte_cursor_from_buf(temp_body_buf),
+                meta_request->checksum_config.checksum_algorithm,
+                aws_byte_cursor_from_buf(&skip_job->part_being_skipped->checksum_base64))) {
             error_code = aws_last_error_or_unknown();
             goto on_done;
         }
@@ -1161,7 +1171,9 @@ static void s_s3_prepare_upload_part_on_read_done(void *user_data) {
 
     /* Reading succeeded. */
     bool is_body_stream_at_end = aws_future_bool_get_result(part_prep->asyncstep2_read_part);
-    request->is_noop = request->request_body.len == 0;
+    request->is_noop = request->part_number >
+                           1 && /* allow first part to have 0 length to support empty unknown content length objects. */
+                       request->request_body.len == 0;
 
     /* If Content-Length is defined, check that we read the expected amount */
     if (has_content_length && (request->request_body.len < request->request_body.capacity)) {
@@ -1177,31 +1189,32 @@ static void s_s3_prepare_upload_part_on_read_done(void *user_data) {
     {
         aws_s3_meta_request_lock_synced_data(meta_request);
 
-        /* TODO: should we signal the client to run update again?
-         * since we just bumped up this number we use to throttle work? */
-        ++auto_ranged_put->synced_data.num_parts_read;
+        --auto_ranged_put->synced_data.num_parts_pending_read;
 
         auto_ranged_put->synced_data.is_body_stream_at_end = is_body_stream_at_end;
 
         if (!request->is_noop) {
             auto_ranged_put->prepare_data.part_index_for_skipping = request->part_number;
 
-            /* Reset values for corresponding checksum and etag.
-             * Note: During resume flow this might cause the values to be
-             * reset twice (if we are preparing part in between
-             * previously completed parts). */
-            struct aws_byte_buf checksum_buf = {0};
-            aws_array_list_set_at(
-                &auto_ranged_put->synced_data.encoded_checksum_list, &checksum_buf, request->part_number - 1);
-
-            struct aws_string *null_etag = NULL;
-            aws_array_list_set_at(&auto_ranged_put->synced_data.etag_list, &null_etag, request->part_number - 1);
+            /* Add part to array-list */
+            struct aws_s3_mpu_part_info *part =
+                aws_mem_calloc(meta_request->allocator, 1, sizeof(struct aws_s3_mpu_part_info));
+            part->size = request->request_body.len;
+            aws_array_list_set_at(&auto_ranged_put->synced_data.part_list, &part, request->part_number - 1);
         }
 
         aws_s3_meta_request_unlock_synced_data(meta_request);
     }
     /* END CRITICAL SECTION */
 
+    /* We throttle the number of parts that can be "pending read"
+     * (e.g. only 1 at a time if reading from async-stream).
+     * Now that read is complete, poke the client to see if it can give us more work.
+     *
+     * Poking now gives measurable speedup (1%) for async streaming,
+     * vs waiting until all the part-prep steps are complete (still need to sign, etc) */
+    aws_s3_client_schedule_process_work(meta_request->client);
+
 on_done:
     s_s3_prepare_upload_part_finish(part_prep, error_code);
 }
@@ -1228,10 +1241,19 @@ static void s_s3_prepare_upload_part_finish(struct aws_s3_prepare_upload_part_jo
             aws_string_c_str(auto_ranged_put->upload_id));
 
     } else {
-        aws_array_list_get_at_ptr(
-            &auto_ranged_put->synced_data.encoded_checksum_list, (void **)&checksum_buf, request->part_number - 1);
-        /* Clean up the buffer in case of it's initialized before and retry happens. */
-        aws_byte_buf_clean_up(checksum_buf);
+
+        /* BEGIN CRITICAL SECTION */
+        {
+            aws_s3_meta_request_lock_synced_data(meta_request);
+            struct aws_s3_mpu_part_info *part = NULL;
+            aws_array_list_get_at(&auto_ranged_put->synced_data.part_list, &part, request->part_number - 1);
+            AWS_ASSERT(part != NULL);
+            checksum_buf = &part->checksum_base64;
+            /* Clean up the buffer in case of it's initialized before and retry happens. */
+            aws_byte_buf_clean_up(checksum_buf);
+            aws_s3_meta_request_unlock_synced_data(meta_request);
+        }
+        /* END CRITICAL SECTION */
 
         AWS_LOGF_DEBUG(
             AWS_LS_S3_META_REQUEST,
@@ -1274,27 +1296,6 @@ static struct aws_future_http_message *s_s3_prepare_complete_multipart_upload(st
 
     struct aws_future_http_message *message_future = aws_future_http_message_new(allocator);
 
-    aws_s3_meta_request_lock_synced_data(meta_request);
-    size_t etag_list_length = aws_array_list_length(&auto_ranged_put->synced_data.etag_list);
-    aws_s3_meta_request_unlock_synced_data(meta_request);
-
-    /* Note: completeMPU fails if no parts are provided. We could
-     * workaround it by uploading an empty part at the cost of
-     * complicating flow logic for dealing with noop parts, but that
-     * arguably adds a lot of complexity for little benefit.
-     * Pre-buffering parts to determine whether mpu is needed will
-     * resolve this issue.
-     */
-    if (!auto_ranged_put->has_content_length && etag_list_length == 0) {
-        AWS_LOGF_ERROR(
-            AWS_LS_S3_META_REQUEST,
-            "id=%p 0 byte meta requests without Content-Length header are currently not supported. Set "
-            "Content-Length header to 0 to upload empty object",
-            (void *)meta_request);
-        aws_future_http_message_set_error(message_future, AWS_ERROR_UNSUPPORTED_OPERATION);
-        return message_future;
-    }
-
     struct aws_s3_prepare_complete_multipart_upload_job *complete_mpu_prep =
         aws_mem_calloc(allocator, 1, sizeof(struct aws_s3_prepare_complete_multipart_upload_job));
     complete_mpu_prep->allocator = allocator;
@@ -1327,8 +1328,6 @@ static struct aws_future_http_message *s_s3_prepare_complete_multipart_upload(st
 static void s_s3_prepare_complete_multipart_upload_on_skipping_done(void *user_data) {
 
     struct aws_s3_prepare_complete_multipart_upload_job *complete_mpu_prep = user_data;
-    struct aws_s3_request *request = complete_mpu_prep->request;
-    struct aws_s3_meta_request *meta_request = request->meta_request;
 
     int error_code = aws_future_void_get_error(complete_mpu_prep->asyncstep_skip_remaining_parts);
     if (error_code != AWS_ERROR_SUCCESS) {
@@ -1336,13 +1335,69 @@ static void s_s3_prepare_complete_multipart_upload_on_skipping_done(void *user_d
         return;
     }
 
-    aws_byte_buf_init(
-        &request->request_body, meta_request->allocator, s_complete_multipart_upload_init_body_size_bytes);
-
     /* Async steps complete, finish up job */
     s_s3_prepare_complete_multipart_upload_finish(complete_mpu_prep, AWS_ERROR_SUCCESS);
 }
 
+/* Allow user to review what we've uploaded, and fail the meta-request if they don't approve. */
+static int s_s3_review_multipart_upload(struct aws_s3_request *request) {
+    struct aws_s3_meta_request *meta_request = request->meta_request;
+    struct aws_s3_auto_ranged_put *auto_ranged_put = meta_request->impl;
+
+    /* If user registered no callback, then success! */
+    if (meta_request->upload_review_callback == NULL) {
+        return AWS_OP_SUCCESS;
+    }
+
+    /* Prepare review info */
+    struct aws_s3_upload_review review = {
+        .checksum_algorithm = meta_request->checksum_config.checksum_algorithm,
+    };
+
+    /* BEGIN CRITICAL SECTION */
+    aws_s3_meta_request_lock_synced_data(meta_request);
+
+    review.part_count = aws_array_list_length(&auto_ranged_put->synced_data.part_list);
+
+    if (review.part_count > 0) {
+        review.part_array =
+            aws_mem_calloc(meta_request->allocator, review.part_count, sizeof(struct aws_s3_upload_part_review));
+
+        for (size_t part_index = 0; part_index < review.part_count; ++part_index) {
+            struct aws_s3_mpu_part_info *part;
+            aws_array_list_get_at(&auto_ranged_put->synced_data.part_list, &part, part_index);
+
+            struct aws_s3_upload_part_review *part_review = &review.part_array[part_index];
+            part_review->size = part->size;
+            part_review->checksum = aws_byte_cursor_from_buf(&part->checksum_base64);
+        }
+    }
+
+    aws_s3_meta_request_unlock_synced_data(meta_request);
+    /* END CRITICAL SECTION */
+
+    /* Invoke callback */
+    int error_code = AWS_ERROR_SUCCESS;
+    if (meta_request->upload_review_callback(meta_request, &review, meta_request->user_data) != AWS_OP_SUCCESS) {
+        error_code = aws_last_error_or_unknown();
+    }
+
+    /* Clean up review info */
+    aws_mem_release(meta_request->allocator, review.part_array);
+
+    if (error_code == AWS_ERROR_SUCCESS) {
+        return AWS_OP_SUCCESS;
+    } else {
+        AWS_LOGF_ERROR(
+            AWS_LS_S3_META_REQUEST,
+            "id=%p: Upload review callback raised error %d (%s)",
+            (void *)meta_request,
+            error_code,
+            aws_error_str(error_code));
+        return aws_raise_error(error_code);
+    }
+}
+
 /* Finish async preparation of a CompleteMultipartUpload request */
 static void s_s3_prepare_complete_multipart_upload_finish(
     struct aws_s3_prepare_complete_multipart_upload_job *complete_mpu_prep,
@@ -1358,8 +1413,23 @@ static void s_s3_prepare_complete_multipart_upload_finish(
     }
 
     AWS_FATAL_ASSERT(auto_ranged_put->upload_id);
-    AWS_ASSERT(request->request_body.capacity > 0);
-    aws_byte_buf_reset(&request->request_body, false);
+
+    if (request->num_times_prepared == 0) {
+        /* Invoke upload_review_callback, and fail meta-request if user raises an error */
+        if (s_s3_review_multipart_upload(request) != AWS_OP_SUCCESS) {
+            error_code = aws_last_error();
+            aws_future_http_message_set_error(complete_mpu_prep->on_complete, error_code);
+            goto on_done;
+        }
+
+        /* Allocate request body */
+        aws_byte_buf_init(
+            &request->request_body, meta_request->allocator, s_complete_multipart_upload_init_body_size_bytes);
+
+    } else {
+        /* This is a retry, reset request body */
+        aws_byte_buf_reset(&request->request_body, false);
+    }
 
     /* BEGIN CRITICAL SECTION */
     aws_s3_meta_request_lock_synced_data(meta_request);
@@ -1371,8 +1441,7 @@ static void s_s3_prepare_complete_multipart_upload_finish(
         meta_request->initial_request_message,
         &request->request_body,
         auto_ranged_put->upload_id,
-        &auto_ranged_put->synced_data.etag_list,
-        &auto_ranged_put->synced_data.encoded_checksum_list,
+        &auto_ranged_put->synced_data.part_list,
         meta_request->checksum_config.checksum_algorithm);
 
     aws_s3_meta_request_unlock_synced_data(meta_request);
@@ -1529,14 +1598,14 @@ static void s_s3_auto_ranged_put_request_finished(
                             AWS_LS_S3_META_REQUEST, "id=%p Failed to parse list parts response.", (void *)meta_request);
                         error_code = AWS_ERROR_S3_LIST_PARTS_PARSE_FAILED;
                     } else if (!has_more_results) {
-                        for (size_t etag_index = 0;
-                             etag_index < aws_array_list_length(&auto_ranged_put->synced_data.etag_list);
-                             etag_index++) {
-                            struct aws_string *etag = NULL;
-                            aws_array_list_get_at(&auto_ranged_put->synced_data.etag_list, &etag, etag_index);
-                            if (etag != NULL) {
+                        for (size_t part_index = 0;
+                             part_index < aws_array_list_length(&auto_ranged_put->synced_data.part_list);
+                             part_index++) {
+                            struct aws_s3_mpu_part_info *part = NULL;
+                            aws_array_list_get_at(&auto_ranged_put->synced_data.part_list, &part, part_index);
+                            if (part != NULL) {
                                 /* Update the number of parts sent/completed previously */
-                                ++auto_ranged_put->synced_data.num_parts_sent;
+                                ++auto_ranged_put->synced_data.num_parts_started;
                                 ++auto_ranged_put->synced_data.num_parts_completed;
                             }
                         }
@@ -1602,13 +1671,14 @@ static void s_s3_auto_ranged_put_request_finished(
                     }
                 }
 
-                struct aws_byte_cursor buffer_byte_cursor = aws_byte_cursor_from_buf(&request->send_data.response_body);
+                struct aws_byte_cursor xml_doc = aws_byte_cursor_from_buf(&request->send_data.response_body);
 
                 /* Find the upload id for this multipart upload. */
-                struct aws_string *upload_id =
-                    aws_xml_get_top_level_tag(meta_request->allocator, &s_upload_id, &buffer_byte_cursor);
+                struct aws_byte_cursor upload_id = {0};
+                const char *xml_path[] = {"InitiateMultipartUploadResult", "UploadId", NULL};
+                aws_xml_get_body_at_path(meta_request->allocator, xml_doc, xml_path, &upload_id);
 
-                if (upload_id == NULL) {
+                if (upload_id.len == 0) {
                     AWS_LOGF_ERROR(
                         AWS_LS_S3_META_REQUEST,
                         "id=%p Could not find upload-id in create-multipart-upload response",
@@ -1618,7 +1688,7 @@ static void s_s3_auto_ranged_put_request_finished(
                     error_code = AWS_ERROR_S3_MISSING_UPLOAD_ID;
                 } else {
                     /* Store the multipart upload id. */
-                    auto_ranged_put->upload_id = upload_id;
+                    auto_ranged_put->upload_id = aws_string_new_from_cursor(meta_request->allocator, &upload_id);
                 }
             }
 
@@ -1712,10 +1782,12 @@ static void s_s3_auto_ranged_put_request_finished(
 
                         ++auto_ranged_put->synced_data.num_parts_successful;
 
-                        /* ETags need to be associated with their part number, so we keep the etag indices
-                         * consistent with part numbers. This means we may have to add padding to the list in the
-                         * case that parts finish out of order. */
-                        aws_array_list_set_at(&auto_ranged_put->synced_data.etag_list, &etag, part_index);
+                        /* Store part's ETag */
+                        struct aws_s3_mpu_part_info *part = NULL;
+                        aws_array_list_get_at(&auto_ranged_put->synced_data.part_list, &part, part_index);
+                        AWS_ASSERT(part != NULL);
+                        AWS_ASSERT(part->etag == NULL);
+                        part->etag = etag;
                     } else {
                         ++auto_ranged_put->synced_data.num_parts_failed;
                         aws_s3_meta_request_set_fail_synced(meta_request, request, error_code);
@@ -1748,8 +1820,7 @@ static void s_s3_auto_ranged_put_request_finished(
                 }
                 /* END CRITICAL SECTION */
 
-                struct aws_byte_cursor response_body_cursor =
-                    aws_byte_cursor_from_buf(&request->send_data.response_body);
+                struct aws_byte_cursor xml_doc = aws_byte_cursor_from_buf(&request->send_data.response_body);
 
                 /**
                  * TODO: The body of the response can be ERROR, check Error specified in body part from
@@ -1759,21 +1830,19 @@ static void s_s3_auto_ranged_put_request_finished(
                  */
 
                 /* Grab the ETag for the entire object, and set it as a header. */
-                struct aws_string *etag_header_value =
-                    aws_xml_get_top_level_tag(meta_request->allocator, &g_etag_header_name, &response_body_cursor);
+                struct aws_byte_cursor etag_header_value = {0};
+                const char *xml_path[] = {"CompleteMultipartUploadResult", "ETag", NULL};
+                aws_xml_get_body_at_path(meta_request->allocator, xml_doc, xml_path, &etag_header_value);
 
-                if (etag_header_value != NULL) {
-                    struct aws_byte_buf etag_header_value_byte_buf;
-                    AWS_ZERO_STRUCT(etag_header_value_byte_buf);
-
-                    aws_replace_quote_entities(meta_request->allocator, etag_header_value, &etag_header_value_byte_buf);
+                if (etag_header_value.len > 0) {
+                    struct aws_byte_buf etag_header_value_byte_buf =
+                        aws_replace_quote_entities(meta_request->allocator, etag_header_value);
 
                     aws_http_headers_set(
                         final_response_headers,
                         g_etag_header_name,
                         aws_byte_cursor_from_buf(&etag_header_value_byte_buf));
 
-                    aws_string_destroy(etag_header_value);
                     aws_byte_buf_clean_up(&etag_header_value_byte_buf);
                 }
 
