@@ -768,7 +768,7 @@ struct aws_s3_meta_request *aws_s3_client_make_meta_request(
             options->checksum_config->checksum_algorithm == AWS_SCA_NONE) {
             AWS_LOGF_ERROR(
                 AWS_LS_S3_CLIENT,
-                "id=%p Cannot create meta s3 request; checksum algorithm must be set to calculate checksum.",
+                "id=%p Cannot create meta s3 request; checksum location is set, but no checksum algorithm selected.",
                 (void *)client);
             aws_raise_error(AWS_ERROR_INVALID_ARGUMENT);
             return NULL;
@@ -777,8 +777,7 @@ struct aws_s3_meta_request *aws_s3_client_make_meta_request(
             options->checksum_config->location == AWS_SCL_NONE) {
             AWS_LOGF_ERROR(
                 AWS_LS_S3_CLIENT,
-                "id=%p Cannot create meta s3 request; checksum algorithm cannot be set if not calculate checksum from "
-                "client.",
+                "id=%p Cannot create meta s3 request; checksum algorithm is set, but no checksum location selected.",
                 (void *)client);
             aws_raise_error(AWS_ERROR_INVALID_ARGUMENT);
             return NULL;
@@ -1694,7 +1693,12 @@ static void s_s3_client_on_acquire_http_connection(
             error_code,
             aws_error_str(error_code));
 
-        if (error_code == AWS_IO_DNS_INVALID_NAME) {
+        if (error_code == AWS_IO_DNS_INVALID_NAME || error_code == AWS_IO_TLS_ERROR_NEGOTIATION_FAILURE) {
+            /**
+             * Fall fast without retry
+             * - Invalid DNS name will not change after retry.
+             * - TLS negotiation is expensive and retry will not help in most case.
+             */
             goto error_fail;
         }
 
@@ -1770,11 +1774,13 @@ void aws_s3_client_notify_connection_finished(
 
         AWS_LOGF_DEBUG(
             AWS_LS_S3_CLIENT,
-            "id=%p Client scheduling retry of request %p for meta request %p with token %p.",
+            "id=%p Client scheduling retry of request %p for meta request %p with token %p with error code %d (%s).",
             (void *)client,
             (void *)request,
             (void *)meta_request,
-            (void *)connection->retry_token);
+            (void *)connection->retry_token,
+            error_code,
+            aws_error_str(error_code));
 
         enum aws_retry_error_type error_type = AWS_RETRY_ERROR_TYPE_TRANSIENT;
 
