@@ -11,13 +11,11 @@
 /* Objects with size smaller than the constant below are bypassed as S3 CopyObject instead of multipart copy */
 static const size_t s_multipart_copy_minimum_object_size = GB_TO_BYTES(1);
 
-static const size_t s_etags_initial_capacity = 16;
-static const struct aws_byte_cursor s_upload_id = AWS_BYTE_CUR_INIT_FROM_STRING_LITERAL("UploadId");
 static const size_t s_complete_multipart_upload_init_body_size_bytes = 512;
 static const size_t s_abort_multipart_upload_init_body_size_bytes = 512;
 
 /* TODO: make this configurable or at least expose it. */
-const size_t s_min_copy_part_size = MB_TO_BYTES(64);
+const size_t s_min_copy_part_size = MB_TO_BYTES(128);
 
 static const struct aws_byte_cursor s_create_multipart_upload_copy_headers[] = {
     AWS_BYTE_CUR_INIT_FROM_STRING_LITERAL("x-amz-server-side-encryption-customer-algorithm"),
@@ -105,7 +103,7 @@ struct aws_s3_meta_request *aws_s3_meta_request_copy_object_new(
     }
 
     aws_array_list_init_dynamic(
-        &copy_object->synced_data.etag_list, allocator, s_etags_initial_capacity, sizeof(struct aws_string *));
+        &copy_object->synced_data.part_list, allocator, 0, sizeof(struct aws_s3_mpu_part_info *));
 
     copy_object->synced_data.content_length = UNKNOWN_CONTENT_LENGTH;
     copy_object->synced_data.total_num_parts = UNKNOWN_NUM_PARTS;
@@ -125,14 +123,15 @@ static void s_s3_meta_request_copy_object_destroy(struct aws_s3_meta_request *me
     aws_string_destroy(copy_object->upload_id);
     copy_object->upload_id = NULL;
 
-    for (size_t etag_index = 0; etag_index < aws_array_list_length(&copy_object->synced_data.etag_list); ++etag_index) {
-        struct aws_string *etag = NULL;
-
-        aws_array_list_get_at(&copy_object->synced_data.etag_list, &etag, etag_index);
-        aws_string_destroy(etag);
+    for (size_t part_index = 0; part_index < aws_array_list_length(&copy_object->synced_data.part_list); ++part_index) {
+        struct aws_s3_mpu_part_info *part = NULL;
+        aws_array_list_get_at(&copy_object->synced_data.part_list, &part, part_index);
+        aws_string_destroy(part->etag);
+        aws_byte_buf_clean_up(&part->checksum_base64);
+        aws_mem_release(meta_request->allocator, part);
     }
 
-    aws_array_list_clean_up(&copy_object->synced_data.etag_list);
+    aws_array_list_clean_up(&copy_object->synced_data.part_list);
     aws_http_headers_release(copy_object->synced_data.needed_response_headers);
     aws_mem_release(meta_request->allocator, copy_object);
 }
@@ -413,6 +412,14 @@ static struct aws_future_void *s_s3_copy_object_prepare_request(struct aws_s3_re
             copy_object->synced_data.total_num_parts = num_parts;
             copy_object->synced_data.part_size = part_size;
 
+            /* Fill part_list */
+            aws_array_list_ensure_capacity(&copy_object->synced_data.part_list, num_parts);
+            while (aws_array_list_length(&copy_object->synced_data.part_list) < num_parts) {
+                struct aws_s3_mpu_part_info *part =
+                    aws_mem_calloc(meta_request->allocator, 1, sizeof(struct aws_s3_mpu_part_info));
+                aws_array_list_push_back(&copy_object->synced_data.part_list, &part);
+            }
+
             AWS_LOGF_DEBUG(
                 AWS_LS_S3_META_REQUEST,
                 "Starting multi-part Copy using part size=%zu, total_num_parts=%zu",
@@ -481,8 +488,7 @@ static struct aws_future_void *s_s3_copy_object_prepare_request(struct aws_s3_re
                 meta_request->initial_request_message,
                 &request->request_body,
                 copy_object->upload_id,
-                &copy_object->synced_data.etag_list,
-                NULL,
+                &copy_object->synced_data.part_list,
                 AWS_SCA_NONE);
 
             break;
@@ -552,19 +558,17 @@ static struct aws_string *s_etag_new_from_upload_part_copy_response(
     struct aws_allocator *allocator,
     struct aws_byte_buf *response_body) {
 
-    struct aws_byte_cursor response_body_cursor = aws_byte_cursor_from_buf(response_body);
+    struct aws_byte_cursor xml_doc = aws_byte_cursor_from_buf(response_body);
+    struct aws_byte_cursor etag_within_xml_quotes = {0};
+    const char *xml_path[] = {"CopyPartResult", "ETag", NULL};
+    aws_xml_get_body_at_path(allocator, xml_doc, xml_path, &etag_within_xml_quotes);
 
-    struct aws_string *etag_within_xml_quotes =
-        aws_xml_get_top_level_tag(allocator, &g_etag_header_name, &response_body_cursor);
-
-    struct aws_byte_buf etag_within_quotes_byte_buf = {0};
-    aws_replace_quote_entities(allocator, etag_within_xml_quotes, &etag_within_quotes_byte_buf);
+    struct aws_byte_buf etag_within_quotes_byte_buf = aws_replace_quote_entities(allocator, etag_within_xml_quotes);
 
     struct aws_string *stripped_etag =
         aws_strip_quotes(allocator, aws_byte_cursor_from_buf(&etag_within_quotes_byte_buf));
 
     aws_byte_buf_clean_up(&etag_within_quotes_byte_buf);
-    aws_string_destroy(etag_within_xml_quotes);
 
     return stripped_etag;
 }
@@ -662,13 +666,14 @@ static void s_s3_copy_object_request_finished(
                     }
                 }
 
-                struct aws_byte_cursor buffer_byte_cursor = aws_byte_cursor_from_buf(&request->send_data.response_body);
+                struct aws_byte_cursor xml_doc = aws_byte_cursor_from_buf(&request->send_data.response_body);
 
                 /* Find the upload id for this multipart upload. */
-                struct aws_string *upload_id =
-                    aws_xml_get_top_level_tag(meta_request->allocator, &s_upload_id, &buffer_byte_cursor);
+                struct aws_byte_cursor upload_id = {0};
+                const char *xml_path[] = {"InitiateMultipartUploadResult", "UploadId", NULL};
+                aws_xml_get_body_at_path(meta_request->allocator, xml_doc, xml_path, &upload_id);
 
-                if (upload_id == NULL) {
+                if (upload_id.len == 0) {
                     AWS_LOGF_ERROR(
                         AWS_LS_S3_META_REQUEST,
                         "id=%p Could not find upload-id in create-multipart-upload response",
@@ -678,7 +683,7 @@ static void s_s3_copy_object_request_finished(
                     error_code = AWS_ERROR_S3_MISSING_UPLOAD_ID;
                 } else {
                     /* Store the multipart upload id. */
-                    copy_object->upload_id = upload_id;
+                    copy_object->upload_id = aws_string_new_from_cursor(meta_request->allocator, &upload_id);
                 }
             }
 
@@ -722,15 +727,11 @@ static void s_s3_copy_object_request_finished(
                     meta_request->progress_callback(meta_request, &progress, meta_request->user_data);
                 }
 
-                struct aws_string *null_etag = NULL;
-                /* ETags need to be associated with their part number, so we keep the etag indices consistent with
-                 * part numbers. This means we may have to add padding to the list in the case that parts finish out
-                 * of order. */
-                while (aws_array_list_length(&copy_object->synced_data.etag_list) < part_number) {
-                    int push_back_result = aws_array_list_push_back(&copy_object->synced_data.etag_list, &null_etag);
-                    AWS_FATAL_ASSERT(push_back_result == AWS_OP_SUCCESS);
-                }
-                aws_array_list_set_at(&copy_object->synced_data.etag_list, &etag, part_index);
+                struct aws_s3_mpu_part_info *part = NULL;
+                aws_array_list_get_at(&copy_object->synced_data.part_list, &part, part_index);
+                AWS_ASSERT(part != NULL);
+                part->etag = etag;
+
             } else {
                 ++copy_object->synced_data.num_parts_failed;
                 aws_s3_meta_request_set_fail_synced(meta_request, request, error_code);
@@ -751,25 +752,21 @@ static void s_s3_copy_object_request_finished(
                  */
                 copy_http_headers(copy_object->synced_data.needed_response_headers, final_response_headers);
 
-                struct aws_byte_cursor response_body_cursor =
-                    aws_byte_cursor_from_buf(&request->send_data.response_body);
+                struct aws_byte_cursor xml_doc = aws_byte_cursor_from_buf(&request->send_data.response_body);
 
                 /* Grab the ETag for the entire object, and set it as a header. */
-                struct aws_string *etag_header_value =
-                    aws_xml_get_top_level_tag(meta_request->allocator, &g_etag_header_name, &response_body_cursor);
-
-                if (etag_header_value != NULL) {
-                    struct aws_byte_buf etag_header_value_byte_buf;
-                    AWS_ZERO_STRUCT(etag_header_value_byte_buf);
-
-                    aws_replace_quote_entities(meta_request->allocator, etag_header_value, &etag_header_value_byte_buf);
+                struct aws_byte_cursor etag_header_value = {0};
+                const char *xml_path[] = {"CompleteMultipartUploadResult", "ETag", NULL};
+                aws_xml_get_body_at_path(meta_request->allocator, xml_doc, xml_path, &etag_header_value);
+                if (etag_header_value.len > 0) {
+                    struct aws_byte_buf etag_header_value_byte_buf =
+                        aws_replace_quote_entities(meta_request->allocator, etag_header_value);
 
                     aws_http_headers_set(
                         final_response_headers,
                         g_etag_header_name,
                         aws_byte_cursor_from_buf(&etag_header_value_byte_buf));
 
-                    aws_string_destroy(etag_header_value);
                     aws_byte_buf_clean_up(&etag_header_value_byte_buf);
                 }
 
