@@ -20,9 +20,9 @@
 #include <aws/io/host_resolver.h>
 #include <aws/io/stream.h>
 #include <aws/io/tls_channel_handler.h>
-#include <aws/testing/async_stream_tester.h>
 #include <aws/testing/aws_test_harness.h>
 #include <aws/testing/stream_tester.h>
+#include <ctype.h>
 #include <inttypes.h>
 #include <stdlib.h>
 #include <time.h>
@@ -252,6 +252,40 @@ static void s_s3_test_meta_request_progress(
     }
 }
 
+static int s_s3_test_meta_request_upload_review(
+    struct aws_s3_meta_request *meta_request,
+    const struct aws_s3_upload_review *review,
+    void *user_data) {
+
+    struct aws_s3_meta_request_test_results *test_results = user_data;
+
+    AWS_FATAL_ASSERT(test_results->upload_review.invoked_count == 0);
+    test_results->upload_review.invoked_count++;
+
+    test_results->upload_review.checksum_algorithm = review->checksum_algorithm;
+
+    test_results->upload_review.part_count = review->part_count;
+    if (test_results->upload_review.part_count > 0) {
+        test_results->upload_review.part_sizes_array =
+            aws_mem_calloc(test_results->allocator, review->part_count, sizeof(uint64_t));
+
+        test_results->upload_review.part_checksums_array =
+            aws_mem_calloc(test_results->allocator, review->part_count, sizeof(struct aws_string *));
+
+        for (size_t i = 0; i < review->part_count; ++i) {
+            test_results->upload_review.part_sizes_array[i] = review->part_array[i].size;
+            test_results->upload_review.part_checksums_array[i] =
+                aws_string_new_from_cursor(test_results->allocator, &review->part_array[i].checksum);
+        }
+    }
+
+    if (test_results->upload_review_callback != NULL) {
+        return test_results->upload_review_callback(meta_request, review, user_data);
+    } else {
+        return AWS_OP_SUCCESS;
+    }
+}
+
 /* Notify the tester that a particular clean up step has finished. */
 static void s_s3_test_client_shutdown(void *user_data);
 
@@ -424,6 +458,9 @@ int aws_s3_tester_bind_meta_request(
     ASSERT_TRUE(options->progress_callback == NULL);
     options->progress_callback = s_s3_test_meta_request_progress;
 
+    ASSERT_TRUE(options->upload_review_callback == NULL);
+    options->upload_review_callback = s_s3_test_meta_request_upload_review;
+
     ASSERT_TRUE(options->user_data == NULL);
     options->user_data = meta_request_test_results;
 
@@ -434,8 +471,8 @@ void aws_s3_meta_request_test_results_init(
     struct aws_s3_meta_request_test_results *test_meta_request,
     struct aws_allocator *allocator) {
 
-    (void)allocator;
     AWS_ZERO_STRUCT(*test_meta_request);
+    test_meta_request->allocator = allocator;
     aws_atomic_init_int(&test_meta_request->received_body_size_delta, 0);
     aws_atomic_init_int(&test_meta_request->total_bytes_uploaded, 0);
     aws_array_list_init_dynamic(
@@ -458,6 +495,12 @@ void aws_s3_meta_request_test_results_clean_up(struct aws_s3_meta_request_test_r
     }
     aws_array_list_clean_up(&test_meta_request->synced_data.metrics);
 
+    for (size_t i = 0; i < test_meta_request->upload_review.part_count; ++i) {
+        aws_string_destroy(test_meta_request->upload_review.part_checksums_array[i]);
+    }
+    aws_mem_release(test_meta_request->allocator, test_meta_request->upload_review.part_sizes_array);
+    aws_mem_release(test_meta_request->allocator, test_meta_request->upload_review.part_checksums_array);
+
     AWS_ZERO_STRUCT(*test_meta_request);
 }
 
@@ -1389,8 +1432,6 @@ int aws_s3_tester_send_meta_request_with_options(
             (meta_request_options.type == AWS_S3_META_REQUEST_TYPE_DEFAULT &&
              options->default_type_options.mode == AWS_S3_TESTER_DEFAULT_TYPE_MODE_PUT)) {
 
-            meta_request_options.checksum_config = &checksum_config;
-
             uint32_t object_size_mb = options->put_options.object_size_mb;
             size_t object_size_bytes = (size_t)object_size_mb * 1024ULL * 1024ULL;
 
@@ -1463,6 +1504,8 @@ int aws_s3_tester_send_meta_request_with_options(
                 .base =
                     {
                         .autogen_length = object_size_bytes,
+                        .eof_requires_extra_read = options->put_options.eof_requires_extra_read,
+                        .max_bytes_per_read = options->put_options.max_bytes_per_read,
                     },
             };
             if (options->put_options.invalid_input_stream) {
@@ -1471,7 +1514,8 @@ int aws_s3_tester_send_meta_request_with_options(
             }
 
             if (options->put_options.async_input_stream) {
-                stream_options.completion_strategy = AWS_ASYNC_READ_COMPLETES_ON_ANOTHER_THREAD;
+                stream_options.completion_strategy = options->put_options.async_read_strategy;
+                stream_options.read_duration_ns = MS_TO_NS(100); /* have async reads take a bit of time. */
 
                 async_stream = aws_async_input_stream_new_tester(allocator, &stream_options);
                 ASSERT_NOT_NULL(async_stream);
@@ -1583,6 +1627,7 @@ int aws_s3_tester_send_meta_request_with_options(
     out_results->body_callback = options->body_callback;
     out_results->finish_callback = options->finish_callback;
     out_results->progress_callback = options->progress_callback;
+    out_results->upload_review_callback = options->upload_review_callback;
 
     out_results->algorithm = options->expected_validate_checksum_alg;
 
@@ -1603,11 +1648,24 @@ int aws_s3_tester_send_meta_request_with_options(
         ASSERT_TRUE(aws_s3_meta_request_is_finished(meta_request));
     }
 
-    aws_s3_tester_lock_synced_data(tester);
+    /* If total_bytes_uploaded wasn't set by the progress callback,
+     * then set it based on number of bytes read from input-stream.
+     * (progress callback isn't currently hooked up for single part upload) */
+    if ((out_results->finished_error_code == AWS_ERROR_SUCCESS) &&
+        (aws_atomic_load_int(&out_results->total_bytes_uploaded) == 0)) {
+
+        uint64_t bytes_read_from_stream = 0;
+        if (input_stream != NULL) {
+            bytes_read_from_stream = aws_input_stream_tester_total_bytes_read(input_stream);
+        } else if (async_stream != NULL) {
+            bytes_read_from_stream = aws_async_input_stream_tester_total_bytes_read(async_stream);
+        }
+        aws_atomic_store_int(&out_results->total_bytes_uploaded, (size_t)bytes_read_from_stream);
+    }
 
     switch (options->validate_type) {
         case AWS_S3_TESTER_VALIDATE_TYPE_EXPECT_SUCCESS:
-            ASSERT_TRUE(out_results->finished_error_code == AWS_ERROR_SUCCESS);
+            ASSERT_INT_EQUALS(AWS_ERROR_SUCCESS, out_results->finished_error_code);
 
             if (meta_request_options.type == AWS_S3_META_REQUEST_TYPE_GET_OBJECT) {
                 ASSERT_SUCCESS(aws_s3_tester_validate_get_object_results(out_results, options->sse_type));
@@ -1625,8 +1683,6 @@ int aws_s3_tester_send_meta_request_with_options(
             break;
     }
 
-    aws_s3_tester_unlock_synced_data(tester);
-
     if (meta_request != NULL) {
         out_results->part_size = meta_request->part_size;
         meta_request = aws_s3_meta_request_release(meta_request);
@@ -1676,7 +1732,7 @@ int aws_s3_tester_send_meta_request(
     if (flags & AWS_S3_TESTER_SEND_META_REQUEST_CANCEL) {
         /* take a random sleep from 0-1 ms. */
         srand((uint32_t)time(NULL));
-        aws_thread_current_sleep(rand() % 1000000);
+        aws_thread_current_sleep(rand() % MS_TO_NS(1));
         aws_s3_meta_request_cancel(meta_request);
     }
 
