@@ -18,6 +18,7 @@
 #include <aws/common/logging.h>
 #include <aws/common/mutex.h>
 #include <aws/common/string.h>
+#include <aws/testing/async_stream_tester.h>
 
 struct aws_client_bootstrap;
 struct aws_credentials_provider;
@@ -162,6 +163,7 @@ struct aws_s3_tester_meta_request_options {
     aws_s3_meta_request_receive_body_callback_fn *body_callback;
     aws_s3_meta_request_finish_fn *finish_callback;
     aws_s3_meta_request_progress_fn *progress_callback;
+    aws_s3_meta_request_upload_review_fn *upload_review_callback;
 
     /* Default Meta Request specific options. */
     struct {
@@ -180,7 +182,13 @@ struct aws_s3_tester_meta_request_options {
         uint32_t object_size_mb;
         bool ensure_multipart;
         bool async_input_stream; /* send via async stream */
-        bool file_on_disk;       /* write to file on disk, then send via aws_s3_meta_request_options.send_filepath */
+        enum aws_async_read_completion_strategy async_read_strategy;
+        size_t max_bytes_per_read; /* test an input-stream read() that doesn't always fill the buffer */
+        bool file_on_disk;         /* write to file on disk, then send via aws_s3_meta_request_options.send_filepath */
+        /* If false, EOF is reported by the read() which produces the last few bytes.
+         * If true, EOF isn't reported until there's one more read(), producing zero bytes.
+         * This emulates an underlying stream that reports EOF by reading 0 bytes */
+        bool eof_requires_extra_read;
         bool invalid_request;
         bool invalid_input_stream;
         bool valid_md5;
@@ -202,12 +210,14 @@ struct aws_s3_tester_meta_request_options {
 
 /* TODO Rename to something more generic such as "aws_s3_meta_request_test_data" */
 struct aws_s3_meta_request_test_results {
+    struct aws_allocator *allocator;
     struct aws_s3_tester *tester;
 
     aws_s3_meta_request_headers_callback_fn *headers_callback;
     aws_s3_meta_request_receive_body_callback_fn *body_callback;
     aws_s3_meta_request_finish_fn *finish_callback;
     aws_s3_meta_request_progress_fn *progress_callback;
+    aws_s3_meta_request_upload_review_fn *upload_review_callback;
 
     struct aws_http_headers *error_response_headers;
     struct aws_byte_buf error_response_body;
@@ -233,6 +243,15 @@ struct aws_s3_meta_request_test_results {
         /* The array_list of `struct aws_s3_request_metrics *` */
         struct aws_array_list metrics;
     } synced_data;
+
+    /* record data from the upload_review_callback */
+    struct {
+        size_t invoked_count;
+        enum aws_s3_checksum_algorithm checksum_algorithm;
+        size_t part_count;
+        uint64_t *part_sizes_array;
+        struct aws_string **part_checksums_array;
+    } upload_review;
 };
 
 struct aws_s3_client_config;
