@@ -1,4 +1,3 @@
-/* crypto/asn1/x_x509.c */
 /* Copyright (C) 1995-1998 Eric Young (eay@cryptsoft.com)
  * All rights reserved.
  *
@@ -68,6 +67,7 @@
 #include <openssl/x509.h>
 #include <openssl/x509v3.h>
 
+#include "../asn1/internal.h"
 #include "../internal.h"
 #include "internal.h"
 
@@ -89,8 +89,6 @@ ASN1_SEQUENCE_enc(X509_CINF, enc, 0) = {
 IMPLEMENT_ASN1_FUNCTIONS(X509_CINF)
 // X509 top level structure needs a bit of customisation
 
-extern void policy_cache_free(X509_POLICY_CACHE *cache);
-
 static int x509_cb(int operation, ASN1_VALUE **pval, const ASN1_ITEM *it,
                    void *exarg) {
   X509 *ret = (X509 *)*pval;
@@ -150,7 +148,6 @@ static int x509_cb(int operation, ASN1_VALUE **pval, const ASN1_ITEM *it,
       ASN1_OCTET_STRING_free(ret->skid);
       AUTHORITY_KEYID_free(ret->akid);
       CRL_DIST_POINTS_free(ret->crldp);
-      policy_cache_free(ret->policy_cache);
       GENERAL_NAMES_free(ret->altname);
       NAME_CONSTRAINTS_free(ret->nc);
       CRYPTO_BUFFER_free(ret->buf);
@@ -329,7 +326,7 @@ int i2d_X509_AUX(X509 *a, unsigned char **pp) {
 }
 
 int i2d_re_X509_tbs(X509 *x509, unsigned char **outp) {
-  x509->cert_info->enc.modified = 1;
+  asn1_encoding_clear(&x509->cert_info->enc);
   return i2d_X509_CINF(x509->cert_info, outp);
 }
 
