@@ -10,9 +10,9 @@ from test import NativeResourceTest
 from concurrent.futures import Future
 
 from awscrt.http import HttpHeaders, HttpRequest
-from awscrt.s3 import S3Client, S3RequestType
+from awscrt.s3 import S3Client, S3RequestType, create_default_s3_signing_config
 from awscrt.io import ClientBootstrap, ClientTlsContext, DefaultHostResolver, EventLoopGroup, TlsConnectionOptions, TlsContextOptions
-from awscrt.auth import AwsCredentialsProvider
+from awscrt.auth import AwsCredentialsProvider, AwsSignatureType, AwsSignedBodyHeaderType, AwsSignedBodyValue, AwsSigningAlgorithm, AwsSigningConfig
 
 MB = 1024 ** 2
 GB = 1024 ** 3
@@ -74,6 +74,7 @@ def s3_client_new(secure, region, part_size=0):
     host_resolver = DefaultHostResolver(event_loop_group)
     bootstrap = ClientBootstrap(event_loop_group, host_resolver)
     credential_provider = AwsCredentialsProvider.new_default_chain(bootstrap)
+    signing_config = create_default_s3_signing_config(region=region, credential_provider=credential_provider)
     tls_option = None
     if secure:
         opt = TlsContextOptions()
@@ -83,7 +84,7 @@ def s3_client_new(secure, region, part_size=0):
     s3_client = S3Client(
         bootstrap=bootstrap,
         region=region,
-        credential_provider=credential_provider,
+        signing_config=signing_config,
         tls_connection_options=tls_option,
         part_size=part_size)
 
@@ -102,12 +103,12 @@ class FakeReadStream(object):
         return fake_data
 
 
-@unittest.skipUnless(os.environ.get('AWS_TEST_S3'), 'set env var to run test: AWS_TEST_S3')
 class S3ClientTest(NativeResourceTest):
 
     def setUp(self):
         self.region = "us-west-2"
         self.timeout = 10  # seconds
+        super().setUp()
 
     def test_sanity(self):
         s3_client = s3_client_new(False, self.region)
@@ -134,9 +135,9 @@ class S3RequestTest(NativeResourceTest):
         self.put_test_object_path = "/put_object_test_py_10MB.txt"
         self.region = "us-west-2"
         self.bucket_name = "aws-crt-canary-bucket"
-        self.default_file_path = "test/resources/s3_put_object.txt"
         self.timeout = 100  # seconds
         self.num_threads = 0
+        self.special_path = "put_object_test_10MB@$%.txt"
         self.non_ascii_file_name = "ÉxÅmple.txt".encode("utf-8")
 
         self.response_headers = None
@@ -149,6 +150,8 @@ class S3RequestTest(NativeResourceTest):
         self.put_body_stream = None
 
         self.files = FileCreator()
+        self.temp_put_obj_file_path = self.files.create_file_with_size("temp_put_obj_10mb", 10 * MB)
+        super().setUp()
 
     def tearDown(self):
         self.files.remove_all()
@@ -221,7 +224,7 @@ class S3RequestTest(NativeResourceTest):
         self._test_s3_put_get_object(request, S3RequestType.GET_OBJECT)
 
     def test_put_object(self):
-        request = self._put_object_request(self.default_file_path)
+        request = self._put_object_request(self.temp_put_obj_file_path)
         self._test_s3_put_get_object(request, S3RequestType.PUT_OBJECT)
         self.put_body_stream.close()
 
@@ -240,6 +243,8 @@ class S3RequestTest(NativeResourceTest):
                 on_headers=self._on_request_headers,
                 on_body=self._on_request_body)
             finished_futures.append(s3_request.finished_future)
+            # request keeps connection alive. delete pointer so connection can shut down
+            del s3_request
         try:
             for future in finished_futures:
                 future.result(self.timeout)
@@ -284,7 +289,7 @@ class S3RequestTest(NativeResourceTest):
             os.remove(file.name)
 
     def test_put_object_file_object(self):
-        request = self._put_object_request(self.default_file_path)
+        request = self._put_object_request(self.temp_put_obj_file_path)
         request_type = S3RequestType.PUT_OBJECT
         # close the stream, to test if the C FILE pointer as the input stream working well.
         self.put_body_stream.close()
@@ -292,7 +297,7 @@ class S3RequestTest(NativeResourceTest):
         s3_request = s3_client.make_request(
             request=request,
             type=request_type,
-            send_filepath=self.default_file_path,
+            send_filepath=self.temp_put_obj_file_path,
             on_headers=self._on_request_headers,
             on_progress=self._on_progress)
         finished_future = s3_request.finished_future
@@ -430,11 +435,55 @@ class S3RequestTest(NativeResourceTest):
         return self._put_object_cancel_helper(False)
 
     def test_multipart_upload_with_invalid_request(self):
-        request = self._put_object_request(self.default_file_path)
+        request = self._put_object_request(self.temp_put_obj_file_path)
         request.headers.set("Content-MD5", "something")
         self._test_s3_put_get_object(request, S3RequestType.PUT_OBJECT, "AWS_ERROR_S3_INVALID_RESPONSE_STATUS")
         self.put_body_stream.close()
 
+    def test_special_filepath_upload(self):
+        # remove the input file when request done
+        with open(self.special_path, 'wb') as file:
+            file.write(b"a" * 10 * MB)
+        request = self._put_object_request(self.special_path)
+        self.put_body_stream.close()
+        s3_client = s3_client_new(False, self.region, 5 * MB)
+        request_type = S3RequestType.PUT_OBJECT
+
+        event_loop_group = EventLoopGroup()
+        host_resolver = DefaultHostResolver(event_loop_group)
+        bootstrap = ClientBootstrap(event_loop_group, host_resolver)
+        credential_provider = AwsCredentialsProvider.new_default_chain(bootstrap)
+        # Let signer to normalize uri path for us.
+        signing_config = AwsSigningConfig(
+            algorithm=AwsSigningAlgorithm.V4,
+            signature_type=AwsSignatureType.HTTP_REQUEST_HEADERS,
+            service="s3",
+            signed_body_header_type=AwsSignedBodyHeaderType.X_AMZ_CONTENT_SHA_256,
+            signed_body_value=AwsSignedBodyValue.UNSIGNED_PAYLOAD,
+            region=self.region,
+            credentials_provider=credential_provider,
+            use_double_uri_encode=False,
+            should_normalize_uri_path=True,
+        )
+
+        s3_request = s3_client.make_request(
+            request=request,
+            type=request_type,
+            send_filepath=self.special_path,
+            signing_config=signing_config,
+            on_headers=self._on_request_headers,
+            on_progress=self._on_progress)
+        finished_future = s3_request.finished_future
+        finished_future.result(self.timeout)
+
+        # check result
+        self.assertEqual(
+            self.data_len,
+            self.transferred_len,
+            "the transferred length reported does not match body we sent")
+        self._validate_successful_get_response(request_type is S3RequestType.PUT_OBJECT)
+        os.remove(self.special_path)
+
     def test_non_ascii_filepath_upload(self):
         # remove the input file when request done
         with open(self.non_ascii_file_name, 'wb') as file:
